package org.yglib.html.parser;

import org.yglib.html.lexer.*;
import org.yglib.html.page.Page;
import org.yglib.html.elements.*;

import java.io.*;
import java.net.URL;
import java.util.*;

/**
 * 
 * @author being20c@gmail.com
 */
public class Parser {
	
	private Lexer lexer = null;
	private Vector<Node> v_Nodes, v_tokens;
	private CheckStack stack = null;
	private IParsingFilter filter = null;
	
	public Parser(File file) throws IOException
	{
		this.lexer = new Lexer(file);
		this.v_Nodes = new Vector<Node>();
	}
	
	public Parser(Page page)
	{
		this.lexer = new Lexer(page);
		this.v_Nodes = new Vector<Node>(2);
		this.v_tokens = new Vector<Node>(5000);
	}
	
	public void setFilter(IParsingFilter filter)
	{
		this.filter = filter;
	}
	
	public Vector<Node> getTokenList()
	{
		return this.v_tokens;
	}
	
	public Node parse()
	{
		this.v_Nodes.clear();
		this.v_tokens.clear();
		
		this.stack = new CheckStack();
		Node node = null;
		while((node = lexer.getNextToken()) != null)
		{
			this.v_tokens.add(node);
			
			if(node instanceof TagNode)
			{
				//System.out.println("NODE>" + node);
				if(this.filter != null)
				{
					if(this.filter.isFiltered(node)) continue;
				}
								
				// TODO DOM    ʿ κ
				TagNode tag = (TagNode)node;
				// Close Tag : CheckStack  
				if(tag.isClosedTag())	
				{
					stack.checkNode2(tag);
				}
				// Open Tag : CheckStatck  Էϰų Tree ȭ
				else	
				{
					// θ node 
					if(this.stack.peek() == null)
					{
						this.v_Nodes.add(tag);
					}
					else
					{
						Node pNode = this.stack.peek();
						tag.setParentNode(pNode);	//  忡 θ 
					}
					
					// stack Է
					if(this.filter != null)
					{
						// End Closed Tag Stack ó  
						if(!tag.isEndClosed() && !this.filter.isFiltered(node)) stack.push(tag);	
					}
					else
					{
						if(!tag.isEndClosed()) stack.push(tag);
					}
				}
			}
			else if(node instanceof TextNode)
			{
				TextNode txNode = (TextNode)node;
				if(this.stack.peek() == null)
				{
					this.v_Nodes.add(txNode);
				}
				else
				{
					Node pNode = this.stack.peek();
					txNode.setParentNode(pNode);	//  忡 θ 
				}
				//System.out.println("text>" + txNode.getParsedText());
			}
			else if(node instanceof CommentNode)
			{
				CommentNode cmNode = (CommentNode)node;
				//System.out.println("comment>" + cmNode.getCommentString());
			}
		}
		System.out.println("Final Stack state..");
		stack.display();
		return null;
	}
	
	public Vector<Node> getParsedList()
	{
		return this.v_Nodes;
	}
	
	public static void main(String...v)
	{
		System.out.println("Parser ..");
		File file = new File("d:\\naverSample.html");
		//File file = new File("d:\\test.html");
		try {
			//URL url_source = new URL("http://news.naver.com/sports/index.nhn?category=baseball&ctg=news&mod=read&office_id=073&article_id=0001965340");
			//http://jakarta.tistory.com/entry/----
			URL url_source = new URL("http://jakarta.tistory.com/entry/----");
			Page page = new Page(url_source, 3000);
			
			Parser parser = new Parser(page);
			parser.parse();
			
			Vector<Node> v_node = parser.getParsedList();
			System.out.println("Parsing Nodes :" + v_node.size());
			for(Node node : v_node)
			{
				if(node instanceof TagNode)
				{
					TagNode tagNode =(TagNode)node;
					System.out.println("TagName >" + tagNode.getTagName());
					dspNode(tagNode);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void dspNode(Node node)
	{
		System.out.println("[NODE]" + node);
		Vector<Node> v_subNodes = node.getSubNodes();
		if(v_subNodes != null)
		{
			for(Node sNode : v_subNodes)
			{
				if(sNode instanceof TagNode)
				{
					TagNode tNode = (TagNode)sNode;
					//System.out.println("TagNode>" + tNode);
					dspNode(sNode);	// recursive print
				}
				else if(sNode instanceof TextNode)
				{
					TextNode tNode = (TextNode)sNode;
					//System.out.println("TextNode>" + tNode.getParsedText());
				}
			}
		}
	}
}
